@@ -1,17 +1,59 @@
 """Generic Trainer and base configuration for model-based agents."""
+from typing import Any
 from typing import List
 from typing import Tuple
 
+from ray.rllib import Policy
 from ray.rllib import SampleBatch
+from ray.rllib.evaluation.worker_set import WorkerSet
 from ray.rllib.utils import override
 
-import raylab.envs as envs
 from raylab.agents import trainer
 from raylab.agents.off_policy import OffPolicyTrainer
 from raylab.utils.annotations import StatDict
 from raylab.utils.replay_buffer import NumpyReplayBuffer
 
 
+def set_policy_with_env_fn(worker_set: WorkerSet, fn_type: str = "reward"):
+    """Set the desired environment function for all policies in the worker set.
+
+    Args:
+        worker_set: A worker set instance, usually from a trainer
+        fn_type: The type of environment function, either 'reward',
+            'termination', or 'dynamics'
+        from_env: Whether to retrieve the function from the environment instance
+            or from the global registry
+    """
+    worker_set.foreach_worker(
+        lambda w: w.foreach_policy(
+            lambda p, _: _set_from_env_if_possible(p, w.env, fn_type)
+        )
+    )
+
+
+def _set_from_env_if_possible(policy: Policy, env: Any, fn_type: str = "reward"):
+    env_fn = getattr(env, fn_type + "_fn", None)
+    if fn_type == "reward":
+        if env_fn:
+            policy.set_reward_from_callable(env_fn)
+        else:
+            policy.set_reward_from_config()
+    elif fn_type == "termination":
+        if env_fn:
+            policy.set_termination_from_callable(env_fn)
+        else:
+            policy.set_termination_from_config()
+    elif fn_type == "dynamics":
+        if env_fn:
+            policy.set_dynamics_from_callable(env_fn)
+        else:
+            raise ValueError(
+                f"Environment '{env}' has no '{fn_type + '_fn'}' attribute"
+            )
+    else:
+        raise ValueError(f"Invalid env function type '{fn_type}'")
+
+
 @trainer.configure
 @trainer.option(
     "holdout_ratio",
@@ -30,7 +72,9 @@ class ModelBasedTrainer(OffPolicyTrainer):
     * Registered via `raylab.envs.register_reward_fn` and
       `raylab.envs.register_termination_fn`
     * Accessible attributes of the environment as `reward_fn` and
-      `termination_fn`
+      `termination_fn`. These should not be bound instance methods; all
+      necessary information should be encoded in the inputs, (state, action,
+      and next state) i.e., the states should be markovian.
 
     Policies must implement `optimize_model` according to
     `raylab.policy:ModelTrainingMixin`
@@ -39,18 +83,8 @@ class ModelBasedTrainer(OffPolicyTrainer):
     @override(OffPolicyTrainer)
     def _init(self, config, env_creator):
         super()._init(config, env_creator)
-        policy = self.get_policy()
-        worker = self.workers.local_worker()
-
-        if envs.has_reward_fn(config["env"]):
-            policy.set_reward_from_config(config["env"], config["env_config"])
-        else:
-            policy.set_reward_from_callable(worker.env.reward_fn)
-
-        if envs.has_termination_fn(config["env"]):
-            policy.set_termination_from_config(config["env"], config["env_config"])
-        else:
-            policy.set_termination_from_callable(worker.env.termination_fn)
+        set_policy_with_env_fn(self.workers, fn_type="reward")
+        set_policy_with_env_fn(self.workers, fn_type="termination")
 
     @staticmethod
     @override(OffPolicyTrainer)
